'use strict';

const node_fs = require('node:fs');
const node_path = require('node:path');
const semver = require('semver');
const consola = require('consola');

function determineSemverChange(commits, config) {
  let [hasMajor, hasMinor, hasPatch] = [false, false, false];
  for (const commit of commits) {
    const semverType = config.types[commit.type]?.semver;
    if (semverType === "major" || commit.isBreaking) {
      hasMajor = true;
    } else if (semverType === "minor") {
      hasMinor = true;
    } else if (semverType === "patch") {
      hasPatch = true;
    }
  }
  return hasMajor ? "major" : hasMinor ? "minor" : hasPatch ? "patch" : null;
}
async function bumpVersion(commits, config, opts = {}) {
  let type = opts.type || determineSemverChange(commits, config) || "patch";
  const originalType = type;
  const pkgPath = node_path.resolve(config.cwd, "package.json");
  const pkg = JSON.parse(await node_fs.promises.readFile(pkgPath, "utf8").catch(() => "{}")) || {};
  const currentVersion = pkg.version || "0.0.0";
  if (currentVersion.startsWith("0.")) {
    if (type === "major") {
      type = "minor";
    } else if (type === "minor") {
      type = "patch";
    }
  }
  if (config.newVersion) {
    pkg.version = config.newVersion;
  } else if (type) {
    pkg.version = semver.inc(currentVersion, type);
    config.newVersion = pkg.version;
  }
  if (pkg.version === currentVersion) {
    return false;
  }
  consola.info(
    `Bumping version from ${currentVersion} to ${pkg.version} (${originalType})`
  );
  await node_fs.promises.writeFile(pkgPath, JSON.stringify(pkg, null, 2) + "\n", "utf8");
  return pkg.version;
}

exports.bumpVersion = bumpVersion;
exports.determineSemverChange = determineSemverChange;
